#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "df.h"
#include "mfsd.h"
#include "inspectHDF.h"

/*
 * FUNCTION: read_ann
 *
 * PURPOSE: READ ANNOTATION that corespond to tag and reference
 *
 * note: only one label and description can correspond to a data set.
 */


void read_ann(hdf_file, tag, reference, settings)
 char *hdf_file;
 uint16 tag, reference;
 settype *settings;
{
  char *ann_label, *ann_desc, *desc_ptr;
  int maxlen;
  int count = 0;

 /*
  * Get annotation label .
  */

  if ((maxlen = DFANgetlablen(hdf_file, tag, reference)) != FAIL)
    {
      printf("  **Annotation Label.\n\n");
      ann_label = (char *) calloc((unsigned) maxlen + 1, (unsigned) sizeof(char));

      if (DFANgetlabel(hdf_file, tag, reference, ann_label, maxlen + 1) != FAIL)
	printf("%s\n\n", ann_label);
      free(ann_label);
    }
  else
    printf("  **No Annotation Label.\n\n");

 /*
  * Get annoation description.
  */

  if ((maxlen = DFANgetdesclen(hdf_file, tag, reference)) != FAIL)
    {
      printf("  **Annotation Description.\n\n");
      ann_desc = (char *) calloc((unsigned) maxlen + 1, (unsigned) sizeof(char));

      if (DFANgetdesc(hdf_file, tag, reference, ann_desc, maxlen + 1) != FAIL)
	{
	  if (!settings->briefANN)
	    printf("%s\n", ann_desc);
	  else
	    {
	      desc_ptr = ann_desc;
	      while ((desc_ptr != ann_desc + maxlen) && (count < settings->briefAMT))
		{
		  printf("%c", *desc_ptr);
		  if (*desc_ptr == '\n')
		    count++;
		  desc_ptr++;
		}
	    }
	}
      free(ann_desc);
    }
  else
    printf("  **No Annotation Description.\n\n");

}				/* END read_ann */
