#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "df.h"
#include "mfsd.h"
#include "inspectHDF.h"


void infoMFSD_VARS();
void view_attributes();
void print_attr_string();
void print_attr_int8();
void print_attr_uint8();
void print_attr_int16();
void print_attr_uint16();
void print_attr_int32();
void print_attr_uint32();
void print_attr_float32();
void print_attr_float64();
void spaces();

/*
 * Function: InfoMFSD
 *
 * Purpose: To display information on MFSDs in a logical presentation
 *      format.
 *
 */

void infoMFSD(hdf_file, settings)
 char *hdf_file;
 settype *settings;
{
  int32 hdf_id, n_datasets, n_global_attributes;

  hdf_id = SDstart(hdf_file, DFACC_RDONLY);

  SDfileinfo(hdf_id, &n_datasets, &n_global_attributes);

  if ((n_datasets > 0) || (n_global_attributes > 0))
    {
      printf("*Multi-file Scientific Data Sets\n");
      printf(" -------------------------------\n\n");

      if (n_global_attributes > 0)
	{
	  printf("\n Global Attributes:\n\n");
	  view_attributes(hdf_id, n_global_attributes, 3);
	  printf("\n\n");
	}

      if (n_datasets > 0)
	{
	  printf("\n Variables:\n\n");
	  infoMFSD_VARS(hdf_id, n_datasets);
	}

    }
  SDend(hdf_id);
}

/*
 * Function: infoMFSD_VARS
 *
 * Purpose: To display all the information on the "variables" or Data Sets
 *	stored in the MFSD.
 */

void infoMFSD_VARS(hdf_id, n_siblings)
 int32 hdf_id;
 int32 n_siblings;
{
  int i, j;
  char name[255];
  int32 id, rank, dimsizes[100], number_type, n_attributes;


  for (i = 0; i < n_siblings; i++)
    {
      id = SDselect(hdf_id, i);

      SDgetinfo(id, name, &rank, dimsizes, &number_type, &n_attributes);

      printf("     %s [", name);
      for (j = 0; j < rank - 1; j++)
	printf("%d, ", dimsizes[j]);
      printf("%d]:", dimsizes[rank - 1]);

      switch (number_type)
	{
	  case DFNT_CHAR8:
	  case DFNT_UCHAR8:
	    printf(" 8-bit character data set.");
	    break;
	  case DFNT_CHAR16:
	  case DFNT_UCHAR16:
	    printf(" 16-bit character data set.");
	    break;
	  case DFNT_INT8:
	    printf(" unsigned 8-bit integer data set.");
	    break;
	  case DFNT_UINT8:
	    printf(" 8-bit integer data set.");
	    break;
	  case DFNT_INT16:
	    printf(" 16-bit integer data set.");
	    break;
	  case DFNT_UINT16:
	    printf(" unsigned 16-bit integer data set.");
	    break;
	  case DFNT_INT32:
	    printf(" 32-bit integer data set.");
	    break;
	  case DFNT_UINT32:
	    printf(" unsigned 32-bit integer data set.");
	    break;
	  case DFNT_INT64:
	    printf(" 64-bit integer data set.");
	    break;
	  case DFNT_UINT64:
	    printf(" unsigned 64-bit integer data set.");
	    break;
	  case DFNT_FLOAT32:
	    printf(" 32-bit floating point data set.");
	    break;
	  case DFNT_FLOAT64:
	    printf(" 64-bit floating point data set.");
	    break;
	  default:
	    printf(" (unknown data type) data set.");
	}

      printf("\n");

      if (n_attributes > 0)
	{
	  printf("\n       attributes:\n");
	  view_attributes(id, n_attributes, 8);
	}
      printf("\n");

      SDendaccess(id);
    }
}

/*
 * Function: view_attributes
 *
 * Purpose: To list all the attributes of a given hdf file or MFSD variable.
 */

void view_attributes(id, n_siblings, indent)
 int32 id;
 int32 n_siblings;
 int indent;
{
  int32 i, n_elements, number_type;
  char name[255];

  for (i = 0; i < n_siblings; i++)
    {
      int start;

      SDattrinfo(id, i, name, &number_type, &n_elements);

     /*
      * Start the indentation process.  The output should look nice :->
      */

      spaces(indent);

     /*
      * Print the name of the "name=value" attribute.
      *
      * "start" is the length of what is printed so proper indentation
      *   can occur with long attributes.
      */

      start = printf("%s = ", name);

     /*
      * Print the value of the "name=value" attribute.  Determine function
      *  to print value based on the data type used to store the information.
      */

      switch (number_type)
	{
	  case DFNT_CHAR8:
	  case DFNT_UCHAR8:
	    print_attr_string(id, i, n_elements);
	    break;
	  case DFNT_INT8:
	    print_attr_int8(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_UINT8:
	    print_attr_uint8(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_INT16:
	    print_attr_int16(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_UINT16:
	    print_attr_uint16(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_INT32:
	    print_attr_int32(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_UINT32:
	    print_attr_uint32(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_FLOAT32:
	    print_attr_float32(id, i, n_elements, start, 12, 75);
	    break;
	  case DFNT_FLOAT64:
	    print_attr_float64(id, i, n_elements, start, 12, 75);
	    break;
	  default:
	    printf("-- Unknown Data Type.  Cannot print vaule. --\n");
	    break;
	}
      printf("\n");
    }
}



/*
 * Functions: print_attr_?????
 *
 * Purpose: To print the value(s) of an attribute in a proper display
 *	format.
 */


void print_attr_string(id, attrnum, n_elements)
 int32 id;
 int32 attrnum;
 int32 n_elements;
{
  char *buffer;

  buffer = (char *) malloc((n_elements + 1) * sizeof(char));

  SDreadattr(id, attrnum, (void *) buffer);

  buffer[n_elements] = '\0';

  printf("%s", buffer);

  free((char *) buffer);
}

void print_attr_int8(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  int8 *buffer;

  col = start;

  buffer = (int8 *) malloc(n_elements * sizeof(int8));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%d, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%d", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_uint8(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  uint8 *buffer;

  col = start;

  buffer = (uint8 *) malloc(n_elements * sizeof(uint8));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%ud, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%ud", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_int16(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  int16 *buffer;

  col = start;

  buffer = (int16 *) malloc(n_elements * sizeof(int16));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%d, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%d", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_uint16(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  uint16 *buffer;

  col = start;

  buffer = (uint16 *) malloc(n_elements * sizeof(uint16));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%ud, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%ud", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_int32(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  int32 *buffer;

  col = start;

  buffer = (int32 *) malloc(n_elements * sizeof(int32));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%ld, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%ld", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_uint32(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  uint32 *buffer;

  col = start;

  buffer = (uint32 *) malloc(n_elements * sizeof(uint32));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%uld, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%uld", buffer[n_elements - 1]);
  free((char *) buffer);
}
void print_attr_float32(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  float32 *buffer;

  col = start;

  buffer = (float32 *) malloc(n_elements * sizeof(float32));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%.2f, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%.2f", buffer[n_elements - 1]);
  free((char *) buffer);
}

void print_attr_float64(id, attrnum, n_elements, start, indent, width)
 int32 id;
 int32 attrnum;
 int32 n_elements;
 int start;
 int indent;
 int width;
{
  int i, col = 0;
  float64 *buffer;

  col = start;

  buffer = (float64 *) malloc(n_elements * sizeof(float64));

  SDreadattr(id, attrnum, (void *) buffer);

  for (i = 0; i < n_elements - 1; i++)
    {
      col += printf("%.2f, ", buffer[i]);

      if (col > width - 10)
	{
	  printf("\n");
	  spaces(indent);
	  col = indent;
	}
    }

  printf("%.2f", buffer[n_elements - 1]);
  free((char *) buffer);
}
