#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "df.h"
#include "mfsd.h"
#include "inspectHDF.h"


/*
 * Currently not in use.  Something I am working on.
 *
 * Should be removed once properly declared in the HDF .h files.
 *
 *   int32 VFnfields();
 *   char * VFfieldname();
 *
 */


/*
 * This order is dependant on the numerical value HDF used to represent this
 *  information.  Currently 0=fully-interlaced, and 1=non-interlaced.  This
 *  order much mirror the VData interlace value.
 */

static char *interlace_msgs[] = {"fully-interlaced", "non-interlaced"};

/*
 * Function: infoVSET
 *
 * Purpose: Get Info on VSETS, if they exists.
 * 
 *
 * PLAN: All VGROUPS with the class "CDF0.0" will be ignored, unless a setting
 *       flag is set.  This way, MFSDs will not be listed in "raw" format.
 *
 *       If new data sets are developed based on VSets, I would recommend
 *       continuing with this approach.  It was my intent for inspectHDF to
 *       be a default high-level viewer with some ability to view at a lower
 *	 level.  Let the good low-level viewer hdfls continue to serve that
 *       function.
 *
 * Bugs: Currently VDatas not attacted to a VGroup are not viewed.
 *
 */

/*
 * Function: infoVSET
 *
 * Purpose: To print the information of VSets in an HDF file.  All VGroups
 * 	that do not have any parents are the starting point to the printout.
 *	All the decendants of these "orphaned" VGroups are printed.
 *
 *      Currently orphaned VDatas are not printed.
 */

void infoVSET(hdf_file, settings)
 char *hdf_file;
 settype *settings;
{
  int i;
  HFILEID hdf_id;
  int32 nentries, vgid, *alone_vgid, alone_size, n_vgids;
  char name[255], class[255];

  hdf_id = Vopen(hdf_file, DFACC_READ, 0);

  n_vgids = Vlone(hdf_id, (int32 *) NULL, (int32) 0);
  alone_size = n_vgids;
  alone_vgid = (int32 *) malloc(n_vgids * sizeof(int32));
  n_vgids = Vlone(hdf_id, alone_vgid, alone_size);

  if (alone_size > 0)
   {
    int not_all_mfsd = 0;

    for (i=0; i<alone_size; i++)
     {
      vgid = Vattach(hdf_id, alone_vgid[i], "r");
      Vinquire(vgid, &nentries, name);
      Vgetclass(vgid, class);
      Vdetach(vgid);

      if (strcmp(class, "CDF0.0") != 0)
	not_all_mfsd = 1;
     }

    if ((not_all_mfsd) || (settings->all_VSETS))
     {
      printf("*VSETS\n");
      printf(" -----\n\n");
     }
   }

  for (i = 0; i < alone_size; i++)
    {
      vgid = Vattach(hdf_id, alone_vgid[i], "r");
      Vinquire(vgid, &nentries, name);
      Vgetclass(vgid, class);

     /*
      * Do not print the VGroup and its decendants used to create a MFSD, 
      * unless explictly requested.
      */

      if ((strcmp(class, "CDF0.0") != 0) || (settings->all_VSETS))
	{
	 /*
	  * Print information on VGroup and look at its decendants.
	  */

	  printf("**VGroup: name:\"%s\"  class:\"%s\"\n", name, class);
	  infoVSET_node(hdf_id, vgid, 1);
	}

      Vdetach(vgid);
    }

  Vclose(hdf_id);
}



void infoVSET_node(hdf_id, parent_vgid, indent)
 HFILEID hdf_id;
 int32 parent_vgid;
 int indent;
{
  int i;
  int32 j;
  int32 childs_id = -1;
  int32 VSele, VSinterlace, VSsize;
  int32 nchildren, ngrandchildren;
  int32 *tag, *ref;
  char name[255], VSfields[255], class[255];

  nchildren = Vntagrefs(parent_vgid);
  tag = (int32 *) malloc(nchildren * sizeof(int32));
  ref = (int32 *) malloc(nchildren * sizeof(int32));
  Vgettagrefs(parent_vgid, tag, ref, nchildren);

  for (i = 0; i < nchildren; i++)
    {
      switch (tag[i])
	{
	  case DFTAG_VG:
	    childs_id = Vattach(hdf_id, ref[i], "r");
	    Vinquire(childs_id, &ngrandchildren, name);
	    Vgetclass(childs_id, class);

	   /*
	    * Print information on VGroup and look at its decendants.
	    */

	    spaces(indent*2);
	    printf("**VGroup: name:\"%s\"  class:\"%s\"\n", name, class);
	    infoVSET_node(hdf_id, childs_id, indent + 1);

	    Vdetach(childs_id);
	    break;
	  case DFTAG_VH:
	    childs_id = VSattach(hdf_id, ref[i], "r");

	    if (VSinquire(childs_id, &VSele, &VSinterlace, VSfields,
			  &VSsize, name) != FAIL)
	      {
	       int32 n_fields, number_type;
	       char *field_name;

		spaces(indent*2);
		printf("**VDATAS:\n");
		spaces(indent*2);
		printf("         Name   : \"%s\"\n", name);
		spaces(indent*2);
		printf("         Fields : %s\n", VSfields);
		spaces(indent*2);
		printf("         Count  : %ld\n", VSele);
		spaces(indent*2);
		printf("         Size   : %ld\n", VSsize);
		spaces(indent*2);
		printf("         Layout : %s\n", interlace_msgs[VSinterlace]);

/* In Progress -- show data type along with field names *
		n_fields = VFnfields(childs_id);
		if (n_fields > 0)
                 {
		  spaces(indent*2);
		  printf("         Fields :\n");
		  for (j=0; j<n_fields; j++)
		    {
		     field_name = VFfieldname(childs_id, i);
		     number_type = VFfieldtype(childs_id, i);
		     spaces(indent*2);
		     printf("                  %s %ld\n", 
			    field_name, number_type);
		    }
		 }
***************/ 

	      }
	    else
	      {
		printf("--- Could not read VDATA components---\n");
	      }
	    VSdetach(childs_id);
	    break;
	  case DFTAG_SD:
	    spaces(indent*2);
	    printf("**SDS Object\n");
	    break;
	  case DFTAG_NT:
	    spaces(indent*2);
	    printf("**Numerical Type Object\n");
	    break;
	  default:			 /* Print the number value of the tag */
	    spaces(indent*2);
	    printf("**Data Object of tag type: %ld.\n", tag[i]);
	}
    }
}
