#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "df.h"
#include "mfsd.h"
#include "inspectHDF.h"

/******************************************************************************
 *
 * Program: inspectHDF
 *
 * Version: 2.0 : unsupported, but comments/suggestions welcomed.
 *
 * Purpose: To display information on HDF files.
 *
 * Compiling Notes:
 *			Written/Compiled on SUN 4.1.3
 *			Compiled on SGI IRIX 4.0.5 System V
 *			Some portability checking with lint.
 *
 * Date: 9/92
 *			Initial development.  Basic intent was a program
 *			for my use only.  Developed a high-level HDF viwer
 *			for the SDS and RIS data sets.
 *
 * Modified: 8/93
 *			Fixed Min/Max problem with SDSs.
 *			Ability to show total number of palettes in the file.	
 *			Better Modularization of the code.
 *                      Incorporation of displaying VSET information.
 *                      Incorporation of displaying MFSD information.
 *
 * Comments:
 *			Output designed modeled after NCSA's Mosaic.
 *			( why change a good thing )
 *
 * Bugs:
 *          Not written in ANSI-C.
 *	    Will only read the first file listed on the command line.
 *          Not all of the return values for error checking from the
 *		HDF routines are monitored.
 *	    Will not list RIS24 data sets.
 *
 * Problems:
 *          Poorly documented, but not horrible.
 *
 * Written By: Neil Buesing
 *	       EROS Data Center
 *	       Sioux Falls, SD 57198
 *
 *	       I am one week from attending grad. school at UIUC, so I do
 *	       not plan on putting on an internet address until I get one
 *	       down there.
 *
 *****************************************************************************/


int main(argc, argv)
 int argc;
 char *argv[];
{
  settype settings;
  char infile[255];

  if (ParseCommandLine(argc, argv, infile, &settings) == -1)
    usage(argv[0]);

  if (!Hishdf(infile))
    {
      printf("\nError: %s is not a HDF file.\n\n", infile);
      exit(-1);
    }

/*
 * Run the desired inspectHDF options.
 */

  if (settings.INTRO)
    infoINTRO(infile);
  if (settings.FILE_INFO)
    infoFILE(infile);
  if (settings.SDS)
    infoSDS(infile, &settings);
  if (settings.RIS8)
    infoRIS(infile, &settings);
  if (settings.PAL)
    infoPAL(infile, &settings);
  if (settings.MFSD)
    infoMFSD(infile, &settings);
  if (settings.VSETS)
    infoVSET(infile, &settings);
  if (settings.INTRO)
    printf("\n* End of InspectHDF.\n\n");

  return (1);

}				/* END MAIN */




void spaces(n)
int n;
{
  int i;

  for (i = 0; i < n; i++)
    printf(" ");
}


/*
 * Function: error.
 *
 * Purpose: Receives message, prints it, and terminates execution 
 *   of the program.
 *
 */

void error(message)
 char *message;
{
  printf("\n%s\n\n", message);
  printf("There may be something wrong with this HDF file.\n\n");
  exit(1);
}

/*
 * Function: ParseCommandLine
 *
 * Purpose: Reads and Parses the command line.  Settings contain the 
 *  stucture to get information from the command line.
 *
 */

int ParseCommandLine(argc, argv, infile, settings)
 int argc;
 char *argv[];
 char *infile;
 settype *settings;
{
  int argindex = 1;

  if (argc < 2)
    return (-1);

  DefaultSettings(settings);

  while (argindex < argc)
    {
      if (argv[argindex][0] == '-')
	switch (argv[argindex][1])
	  {
	    case 'h':
	      return (-1);	/* ParseCommandLine fails and usage is printed*/
	    case 'b':
	      settings->briefANN = 1;	/* only print x number of lines; */
	      settings->briefAMT = atoi(argv[argindex + 1]);
	      argindex++;
	      break;
	    case 'n':
	      switch (argv[argindex][2])
		{
		  case 's':
		    settings->SDS = 0;
		    break;
		  case 'r':
		    settings->RIS8 = 0;
		    break;
		  case 'p':
		    settings->PAL = 0;
		    break;
		  case 'v':
		    settings->VSETS = 0;
		    break;
		  case 'm':
		    settings->MFSD = 0;
		    break;
		  case 'f':
		    settings->FILE_INFO = 0;
		    break;
		  case 'i':
		    settings->INTRO = 0;
		    break;
		}
	      break;
	    case 'v':
	      switch (argv[argindex][2])
		{
		  case 'a':
		    settings->all_VSETS = 1;
		    break;
		}
	  }
      else
	{
	  if (argindex < argc)
	    (void) strcpy(infile, argv[argindex]);
	  else
	    return (-1);
	}
      argindex++;
    }
  return (0);
}


/*
 * Function: DefaultSettings
 *
 * Purpose: Options used if not specified in command line.
 *
 */

void DefaultSettings(settings)
 settype *settings;
{
  settings->briefANN = 0;	/* default is all of an annotation */
  settings->briefAMT = 0;
  settings->SDS = 1;		/* default is to include every option */
  settings->MFSD = 1;
  settings->RIS8 = 1;
  settings->PAL = 1;
  settings->VSETS = 1;
  settings->FILE_INFO = 1;
  settings->all_VSETS = 0;
  settings->INTRO = 1;
}


/*
 * Function: usage.
 *
 * Purpose: prints command line structure when it is incorrect or reqested.
 *
 */

void usage(pmg)
 char *pmg;
{
  printf("\nUSAGE: %s [options] infile \n\n", pmg);
  printf("       options:\n");
  printf("               -h     this help screen.\n");
  printf("               -b #   abrv. data object annotation descriptions.\n");
  printf("               -ni    don't print intro section.\n");
  printf("               -nf    don't list information on File Annotations.\n");
  printf("               -nm    don't list information on MFSD.\n");
  printf("               -nr    don't list information on RIS8.\n");
  printf("               -np    don't list information on Palettes.\n");
  printf("               -ns    don't list information on SDS.\n");
  printf("               -nv    don't list information on VSETS.\n");
  printf("               -va    list all VSETS (including MFSDs).\n");
  printf("\n");
  exit(-1);
}
